<table width="100%" cellspacing="1" class="item">
 <tr class="item">
  <th class="item" align="left"><?php echo _("Title") ?></th>
  <th class="item" align="left"><?php echo _("Location") ?></th>
  <th class="item" align="left"><?php echo _("Status") ?></th>
  <th class="item" align="left"><?php echo _("Start On") ?></th>
  <th class="item" align="left"><?php echo _("End On") ?></th>
 </tr>
