<br class="spacer" />
<div class="header">
 <span class="rightFloat">
  <?php echo count($events) == 1 ? _("1 event") : sprintf(_("%d events"), count($events)) ?>
 </span>
 <?php echo _("Search Results") ?>
</div>
