<table id="kronolithViewDay" class="kronolith-view kronolithViewDay<?php if ($this->sidebyside) echo ' kronolith-sidebyside' ?>">
  <thead class="kronolith-view-head"><tr>
    <th class="kronolith-first-col">&nbsp;</th>
    <th colspan="<?php echo $this->totalspan ?>"><?php echo _("Events for this day") ?></th>
  </tr></thead>
