.kronolith-first-col span, .kronolithViewBody .kronolith-first-col span {
    filter: progid:DXImageTransform.Microsoft.BasicImage(rotation=3);
    height: 20px;
    width: 40px;
    margin-bottom: 20px;
}
.kronolith-all-day .kronolith-first-col span {
    width: 80px;
    margin-bottom: 60px;
}
#kronolithLoadingyear {
    left: -26px;
}
