 <tr>
  <td class="label">&nbsp;</td>
  <td class="label" colspan="<tag:span />">&nbsp;</td>
 </tr>
 <tr>
  <td class="label">&nbsp;</td>
  <td class="label" colspan="<tag:span />">
   <span style="padding-left:15px;" class="kronolithFBBusy">&nbsp;</span>
   <span class="kronolithFBLegend"><gettext>Busy</gettext></span>
   <span style="padding-left:15px;" class="kronolithFBFree">&nbsp;</span>
   <span class="kronolithFBLegend"><gettext>Possible Meeting Time</gettext></span>
   <span style="padding-left:15px;" class="kronolithFBUnknown">&nbsp;</span>
   <span class="kronolithFBLegend"><gettext>No Information</gettext></span>
  </td>
 </tr>
