/* Navigation bar */
.horde-buttonbar a.kronolithNext,
.horde-buttonbar a.kronolithPrev {
    color: transparent;
    padding-right: 18px;
    text-indent: -10000px;
}
.horde-buttonbar a.kronolithNext {
    background-image: url("graphics/right.png");
}
.horde-buttonbar a.kronolithPrev {
    background-image: url("graphics/left.png");
}
#kronolithNavTasks {
    background-image: url("graphics/tasks.png");
}
#kronolithNavAgenda {
    /*
    background-image: url("graphics/agenda.png");
    */
}
#kronolithNavDay {
    background-image: url("graphics/dayview.png");
}
#kronolithNavWeek {
    background-image: url("graphics/weekview.png");
}
#kronolithNavWorkweek {
    background-image: url("graphics/workweekview.png");
}
#kronolithNavMonth {
    background-image: url("graphics/monthview.png");
}
#kronolithNavYear {
    background-image: url("graphics/yearview.png");
}

/* Autocomplete styles. */
.autocomplete {
    background: #f4f4f4;
    border: 1px solid #d4d4d4;
    position: absolute;
    padding: 2px 2px 2px 2px;
}
.autocomplete ul {
    margin: 1px;
    padding: 1px;
    list-style-type: none;
}
.autocomplete ul li {
    padding-top: 3px;
    padding-bottom: 3px;
    cursor: pointer;
    text-decoration: underline;
    font-size: 90%;
    color: #000;
}
.autocomplete ul .selected {
    background: gray;
    color: #fff;
}

/* Calendar styles. */
table.kronolith-view {
    width: 100%;
    height: 100%;
    table-layout: fixed;
    background-color: #dadada;
    border-spacing: 1px;
}
table.kronolith-view.kronolith-sidebyside {
    width: auto;
}
table.kronolith-view tr {
    background-color: #c2cccf;
}
table.kronolith-view td, table.kronolith-view th {
    padding: 0;
    vertical-align: top;
    overflow: hidden;
}
.kronolith-view-head table td, table.kronolith-view-head td,
.kronolith-view-head th, table .kronolith-view-head th {
    height: 21px;
    font-weight: normal;
    text-align: center;
    text-overflow: ellipsis;
    -o-text-overflow: ellipsis;
    vertical-align: middle;
}
.kronolith-sidebyside .kronolith-view-head table td,
.kronolith-sidebyside table.kronolith-view-head td,
.kronolith-sidebyside .kronolith-view-head th,
.kronolith-sidebyside table .kronolith-view-head th {
    padding: 0 10px;
}
.kronolith-view-head table td div, table.kronolith-view-head td div {
    text-align: left;
}
.kronolith-first-col {
    text-align: center;
}
table.kronolith-view td.kronolith-first-col,
table.kronolith-view th.kronolith-first-col {
    width: 21px;
    background-color: #c2cccf;
    vertical-align: middle;
/*    padding: 9px 0 0;*/
}
.kronolith-all-day td.kronolith-first-col span {
    margin-top: 32px;
}
#kronolith-month-body td.kronolith-first-col {
    vertical-align: top;
    padding-top: 7px;
}
table.kronolith-view.kronolith-sidebyside td.kronolith-first-col {
    padding-top: 3px;
}
table.kronolith-view.kronolith-sidebyside .kronolith-all-day td.kronolith-first-col {
    padding-top: 5px;
}
.kronolith-first-col span {
    display: block;
    margin-top: 10px;
    white-space: nowrap;
    transform: rotate(-90deg);
    -webkit-transform: rotate(-90deg);
    -moz-transform: rotate(-90deg);
}
.kronolithViewMonth.kronolith-sidebyside .kronolith-first-col {
    padding: 7px;
    text-align: left;
}
.kronolithViewBody table.kronolith-view tr,
.kronolith-all-day td,
#kronolith-month-body tr {
    background-color: #fff;
}
#kronolith-month-body tr {
    height: 100px;
}
table.monthgrid {
    width: 100%;
    border-top: 1px solid #999;
    border-left: 1px solid #999;
}
table.monthgrid th, table.monthgrid td {
    border-right: 1px solid #999;
    border-bottom: 1px solid #999;
}
.kronolith-other-month {
    background: #f7f7f7;
}
.kronolith-other-month .kronolith-day {
    border-color: transparent;
    color: silver;
}
.kronolith-weekend, .kronolithViewDay .kronolith-weekend {
    background: #ffc;
}
.kronolith-today, .kronolithViewDay .kronolith-today {
    background-color: #7eaa42;
}
.description {
    padding: .5em;
}
.kronolith-day {
    height: 18px;
    padding: 7px 0 0 10px;
}
.kronolith-today .kronolith-day, .kronolith-today .kronolith-day a {
    color: #fff;
}

.kronolithViewDay td {
    height: 40px;
    background-color: #fff;
}

table .kronolith-all-day td {
    height: 80px;
}

table.kronolithViewMonth {
    border: 1px solid #a6afb1;
    border-top-width: 0;
}

.kronolith-minical {
    border-spacing: 10px;
}
.kronolith-minical td {
    background-color: #d5dcde;
    padding: 10px;
    vertical-align: top;
}

/* Mini calendar */
.kronolith-minical table {
    border-spacing: 1px;
    border: 0;
    background-color: #d5dcde;
}
.kronolith-minical table th, .kronolith-minical table td {
    background-color: #fff;
    height: 22px;
    width: 22px;
    padding: 1px;
    text-align: center;
    vertical-align: middle;
}
.kronolith-minical table th,
.kronolith-minical table td.kronolith-minical-week {
    background-color: #f2f2f2;
    color: #6d6d6d;
}
.kronolith-minical table tbody td {
    empty-cells: show;
}
.kronolith-minical table tbody td a {
    display: block;
}
.kronolith-minical table tbody td:hover,
.kronolith-minical table tbody td:hover a,
 kronolith-minical .kronolith-selected:hover,
#horde-sidebar .kronolith-minical .kronolith-selected:hover a {
    background-color: #f1f1f1;
    color: #000;
}
tr.kronolith-minical-nav, tr.kronolith-minical-nav th {
    background-color: #d5dcde;
}
#kronolith-minical-prev, #kronolith-minical-next {
    display: block;
    text-indent: -10000px;
    cursor: pointer;
    background: transparent url("graphics/right.png") no-repeat center center;
}
#kronolith-minical-prev {
    background-image: url("graphics/left.png");
}
.kronolith-minical .kronolith-minical-empty {
    background: none;
}
.kronolith-minical .kronolith-other-month,
.kronolith-minical .kronolith-other-month a {
    color: #6d6d6d;
}
.kronolith-minical .kronolith-selected,
#horde-sidebar .kronolith-minical .kronolith-selected a {
    background-color: #36525d;
    color: #fff;
}
.kronolith-minical .kronolith-today,
.kronolith-minical .kronolith-today:hover,
.kronolith-minical .kronolith-today a,
.kronolith-minical .kronolith-today:hover a,
#horde-sidebar .kronolith-minical .kronolith-today a {
    background-color: #7eaa42 !important;
    color: #fff;
}
.kronolith-minical .kronolith-selected.kronolith-today {
    border-color: #fff;
}

/* Events. */
.kronolith-event {
    overflow: hidden;
    text-overflow: ellipsis;
    -o-text-overflow: ellipsis;
}
.kronolith-event .kronolith-event-info {
    padding: 3px 5px 4px 5px;
    overflow: hidden;
    line-height: 16px;
    text-overflow: ellipsis;
    -o-text-overflow: ellipsis;
    text-decoration: inherit;
}
img.kronolithEventIcon {
    float: left;
    margin: 0 2px 2px 0;
}
.kronolith-event .kronolith-time {
    font-weight: bold;
    margin-right: 2px;
    white-space: nowrap;
}
.kronolith-event-tentative {
    font-style: italic;
}
.kronolith-event-cancelled {
    text-decoration: line-through;
}
.kronolith-location {
    font-size: 90%;
    margin-left: 2px;
}
.kronolith-event img {
    padding-top: 3px;
    vertical-align: top;
}
#kronolithViewMonth .kronolith-event,
.kronolithViewDay .kronolith-all-day .kronolith-event {
    line-height: 15px;
    padding: 3px 10px;
}
#kronolithViewMonth .kronolith-event {
    max-height: 2.5em;
}
.kronolithViewDay .kronolith-event {
    max-height: 40px;
}
#kronolithViewMonth .kronolith-event:hover,
.kronolithViewDay .kronolith-event:hover {
    max-height: none;
}
#kronolithViewDay .kronolith-all-day .kronolith-event {
    float: left;
}

/* Event form. */
#eventform .control {
    height: 22px;
    padding: 0 10px;
}
#eventform textarea {
    padding: 2px;
    width: 98%;
}
#eventform .toggle {
    cursor: pointer;
}
#eventform .extra {
    color: #666;
}
#eventform .warning, #deleteform .warning {
    background: #ffc;
    border: 1px solid #fff760;
    padding: 8px;
}
#eventform #start_img, #eventform #end_img, #eventform #recur_end_img {
    cursor: pointer;
}
#eventform .horde-form-buttons {
    padding-left: 0;
}

/* background-color will always be overridden for the
 * *-eventbox classes */
.block-eventbox, table.nopadding td.block-eventbox {
    padding: 0 4px;
}

.selected-control {
    background: #bbcbff;
    border-bottom: 1px solid #999;
}

a.newEvent img {
    vertical-align: top;
    padding: 3px 0 0 3px;
}

.year-event {
    background: #fff760;
}

.kronolithFBLegend {
    font-size: 90%;
    padding-right: 10px;
}
.kronolithFBBusy {
    position: relative;
    background: #f03232;
}
.kronolithFBUnknown {
    background: #d4d0c8 url("graphics/unknown-background.png") repeat left top;
}
.kronolithFBFree {
    position: relative;
    background: #8fc34b;
    color: #fff;
}
div.fbgrid {
    background-color: #fff;
    overflow: auto;
    padding: 0 1ex 0 1ex;
}
.fbgrid table {
    width: 100%;
    border-collapse: collapse;
}
.fbgrid table td, .fbgrid table th {
    border: 1px solid #000;
}
.fbgrid table .label {
    border: none;
    padding-right: 1ex;
}

/* Icons. */
.kronolith-icon-today {
    background-image: url('graphics/today.png');
}
.kronolith-icon-goto {
    background-image: url('graphics/goto.png');
}
.kronolith-icon-search {
    background-image: url('graphics/search.png');
}
.kronolith-icon-back {
    background-image: url('graphics/back.png');
}

/* Tags */
.panel-tags {
    margin-left: 10px;
}
.panel-tags ul {
    margin: 1px;
    padding: 1px;
    list-style-type: none;
}
li.panel-tags {
    float: left;
    padding: 2px;
}
.panel-tags li:hover {
    text-decoration: underline;
}

#color-picker {
    z-index: 103;
}

#calendarPrint {
    display: none;
}

/* Prefs */
.fburl {
    border: 1px solid black;
    padding: 5px 0 10px 15px;
}
.fburl .fixed {
    padding-top: 3px;
}

@media print {
    .rowEven td, .rowOdd td {
        background: #fff;
        color: #000;
        border-bottom: thin solid #000;
    }
    .rowEven td td, .rowOdd td td {
        border: none;
    }
    .selected-control {
        background: #fff;
        border-bottom: thin solid #000;
    }

    .iconAlarm, .iconDelete, .iconEdit, .iconRecur, .iconAdd, .iconNav,
    a.newEvent, .tabset {
        display: none;
    }

    #calendarPrint {
        display: block;
    }

    .year-event {
        background: #fff;
    }
    table.monthgrid {
        border-top: thin solid #000;
        border-left: thin solid #000;
    }
    table.monthgrid th, table.monthgrid td {
        border-right: thin solid #000;
        border-bottom: thin solid #000;
        padding: 2px;
    }
    .month-eventbox {
        border: none;
        border-bottom: thin solid #000;
    }

    .kronolith-all-day td {
        height: auto;
    }

    .kronolith-today {
        background-color: inherit;
    }
}
