.kronolith_embedded {
    margin: 0;
    padding: 0;
    font-size: 90%;
}

.kronolith_embedded a {
    color: blue;
    text-decoration: none;
}
.kronolith_embedded a:hover {
    text-decoration: underline;
}

.kronolith_embedded .title {
    font-weight: bold;
    font-size: 100%;
}

.kronolith_embedded .control {
    background: #CCCCCC none repeat scroll 0 0;
    border-bottom: 1px solid #999999;
    color: #000000;
    padding: 1px;
}

.kronolith_embedded .linedRow, .kronolith_embedded tr.linedRow td,
.kronolith_embedded table.linedRow td, .kronolith_embedded ul.linedRow li {
    background: #FFFFFF none repeat scroll 0 0;
    border-bottom: 1px solid #DDDDDD;
    color: #000000;
}

.kronolith_embedded .nowrap {
    white-space: nowrap;
}

.kronolith_embedded .text {
    background: #FFFFFF none repeat scroll 0 0;
    color: #000000;
}

.kronolith_embedded .block-eventbox,
.kronolith_embedded table.nopadding td.block-eventbox {
    padding: 0 4px;
}

.kronolith_embedded .event {
}

.kronolith_embedded img, .kronolith_embedded .img,
.kronolith_embedded .img:active, .kronolith_embedded .img:hover,
.kronolith_embedded .img:visited, .kronolith_embedded .image {
    background: transparent none repeat scroll 0 0;
    border: medium none;
    vertical-align: middle;
}

.kronolith_embedded  .block-monthgrid {
    background: #FFFFFF none repeat scroll 0 0;
}

.kronolith_embedded .item {
    background: #EEEEEE none repeat scroll 0 0;
    color: #000000;
}
.kronolith_embedded .kronolith-othermonth {
    background: #f7f7f7;
}
.kronolith_embedded .block-monthgrid .today {
    border: 1px solid #444466;
}
.kronolith_embedded .kronolith-weekend {
    background: #ffc;
}

.kronolith_embedded .clear {
    clear: both;
    height: 0;
    line-height: 0;
}

.kronolith_embedded table {
    border: medium none;
}

.kronolith_embedded .kronolith-event-cancelled {
    text-decoration: line-through;
}

.kronolith_embedded td, .kronolith_embedded th {
    padding: 1px;
}
div.horde_nicetitle {
    color: #000;
    padding: 5px;
    z-index: 1001;
    position: absolute;
    overflow: hidden;
    background: #eceff0;
    border: 1px solid #919293;
    -moz-box-shadow: 3px 3px 3px #888;
    -webkit-box-shadow: 3px 3px 3px #888;
    box-shadow: 3px 3px 3px #888;
}
div.horde_nicetitle pre {
    text-align: left;
    font-size: 75%;
    font-family: "Lucida Console",Courier,"Courier New";
}
