<br />
<table class="nowrap" cellspacing="0" width="100%">
 <tr>
  <td class="label">&nbsp;</td>
  <td class="label" colspan="<tag:span />"><strong><tag:title /></strong></td>
 </tr>
 <tr>
  <td class="label">&nbsp;</td>
  <tag:hours />
 </tr>
 <tag:rows />
 <tag:legend />
</table>
