.monthgrid {
    background: #fff;
}
.upcomingday td {
    border-top: 1px solid #ddd;
}

.kronolith-today {
    background-color: #7eaa42;
}
.kronolith-weekend {
    background-color: #ffc;
}

.kronolith-event-tentative {
    font-style: italic;
}
.kronolith-event-cancelled {
    text-decoration: line-through;
}
